require 'gyoku'
require 'egon_gate/egsb/config'

module EgonGate
  module Egsb
    class Message

      class_attribute :attributes

      def config
        @config ||= EgonGate::Egsb::Config.new
      end

      def message_id
        @message_id ||= SecureRandom.uuid
      end

      def message_code
        raise NotImplementedError.new
      end

      def message_tag
        raise NotImplementedError.new
      end

      def message_request_tag
        raise NotImplementedError.new
      end

      def header
        {
          'reg:CasZadosti' => Time.now.strftime('%Y-%m-%dT%H:%M:%S.%L%:z'),
          'reg:Agenda' => config.agenda,
          'reg:AgendovaRole' => config.role_of_agenda,
          'reg:Ovm' => config.ovm_code,
          'reg:Ais' => config.ais_code,
          'reg:Uzivatel' => 'OEzr',
          'reg:AgendaZadostId' => message_id
        }
      end

      def namespace_definitions
        {
          'xmlns:abs' => 'urn:cz:isvs:iszr:schemas:IszrAbstract:v1',
          'xmlns:reg' => 'urn:cz:isvs:reg:schemas:RegTypy:v1'
        }
      end

      def message_body
        {}
      end


      def message
        {
          message_tag => {
            'abs:ZadostInfo' => header,
            'nsgon:Zadost' => {
              message_request_tag => message_body
            }
          },
          attributes!: { message_tag => namespace_definitions }
        }
      end

      def parse_response(response)
        response[:ruian_vyhledej_adresu_response][:ruian_odpoved][:ruian_vyhledej_adresu_data_response][:adresy][:adresa]
      end

      def to_s
        Gyoku.xml(message, key_converter: :camelcase)
      end

    end
  end
end
